import click
import yaml
import re
import json
import os
import time
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def get_namespace():
    # Get domain
    domains = settings.ORDERER_DOMAINS.split(' ')
    if len(domains) == 0:
        domains = settings.PEER_DOMAINS.split(' ')
    explorer_namespace = domains[0]
    
    # Create temp folder & namespace
    settings.k8s.prereqs(explorer_namespace)
    return explorer_namespace

def generate_prom_config():
    prom_target = ''
    # Get orderer target
    orderers = settings.ORDERER_ORGS.split(' ')
    for orderer in orderers:
        domain = util.get_domain(orderer)
        for peer in range(int(settings.NUM_ORDERERS)):
            if len(prom_target) > 0:
                prom_target += ','
            prom_target += '\'orderer%s-%s.%s:10443\'' % (peer, orderer, domain)

    # Get peer target
    peerorgs = settings.PEER_ORGS.split(' ')
    for peerorg in peerorgs:
        domain = util.get_domain(peerorg)
        for peer in range(int(settings.NUM_PEERS)):
            if len(prom_target) > 0:
                prom_target += ','
            prom_target += '\'peer%s-%s.%s:10443\'' % (peer, peerorg, domain)
    dict_env = {
        'PROM_TARGET': prom_target
    }

    # Load config
    config_template = '%s/prometheus/prometheus-template.yml' % util.get_k8s_template_path()
    yaml_path, config = util.load_yaml_config_template(
        yaml_template_path=config_template, dict_env=dict_env)

    print('config: ', config)
    return yaml_path

def cp_config_efs(yaml_path):
    # Find efs pod
    pods = settings.k8s.find_pod(namespace="default", keyword="test-efs")
    if not pods:
        return hiss.hiss('cannot find tiller pod')

    # Create folder config
    exec_command = [
        '/bin/bash',
        '-c',
        'mkdir -p %s/promconfig' % (settings.EFS_ROOT)]
    result_create_folder = settings.k8s.exec_pod(
        podName=pods[0], namespace="default", command=exec_command)
    if result_create_folder.success == False:
        return hiss.hiss('cannot remove folders in %s' % pods[0])

    # Copy to efs pod
    target_file = '%s/promconfig/prometheus.yml' % settings.EFS_ROOT
    if not settings.k8s.cp_to_pod(podName=pods[0], namespace='default', source=yaml_path, target=target_file):
        return hiss.hiss('connot copy test chaincode to pod %s' % pods[0])

    return True

def setup_prometheus():

    hiss.echo('Generate prometheus config')
    config_path = generate_prom_config()
    if not cp_config_efs(config_path):
        return util.Result(success=False, msg='Cannot copy prom config to efs pod')

    hiss.echo('Deploy prometheus')
    # Get domain
    prom_namespace = get_namespace()
    
    # Create temp folder & namespace
    settings.k8s.prereqs(prom_namespace)

    dict_env = {
        'DOMAIN': prom_namespace,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND
    }

    # Deploy prometheus sts
    prom_template = '%s/prometheus/prometheus-stateful.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=prom_namespace, k8s_template_file=prom_template, dict_env=dict_env)
    prom_svc_template = '%s/prometheus/prometheus-service-stateful.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=prom_namespace, k8s_template_file=prom_svc_template, dict_env=dict_env)

def del_prometheus():
    # Delete prom
    return settings.k8s.delete_stateful(name='prometheus', namespace=get_namespace())

@click.group()
def prometheus():
    """Prometheus"""
    pass

@prometheus.command('setup', short_help="Setup prometheus")
def setup():

    hiss.rattle('Setup prometheus')
    setup_prometheus()
    


@prometheus.command('delete', short_help="Delete prometheus")
def delete():

    hiss.rattle('Delete prometheus')
    del_prometheus()



# Settings module contains all global objects that are shared accross the mamba project
# It is useful for utility objects that only need to be initialized once
# follow the instruction from here:
# https://stackoverflow.com/questions/13034496/using-global-variables-between-files
import os
from os.path import expanduser
import yaml
import shutil
from utils.kube import KubeHelper
from dotenv import load_dotenv
from utils import util
import git
import subprocess

def init(dotenv_path, set_default):
    default_path = expanduser('~/.akachain/akc-mamba/mamba/config/.env')
    if set_default:
        shutil.copy(dotenv_path, default_path)
        load_dotenv(default_path)
    else:
        print('Loading config from akachain git repo...')
        # clone repo akc-mamba to load config
        mamba_path = expanduser('~/.akachain')
        if not os.path.isdir(mamba_path):
            os.makedirs(mamba_path)
            git.Git(mamba_path).clone('https://github.com/Akachain/akc-mamba.git', branch='binary-config')
            env_template_path = expanduser('~/.akachain/akc-mamba/mamba/config/operator.env-template')
            shutil.copy(env_template_path, default_path)
            bashCommand = 'sudo vi ' + default_path
            subprocess.call(bashCommand, shell=True)

        print("%s", dotenv_path)
        load_dotenv(dotenv_path)

    global PVS_PATH
    global K8S_TYPE
    K8S_TYPE = os.getenv('K8S_TYPE')
    if K8S_TYPE == 'eks':
        PVS_PATH = '/pvs'
    else:
        PVS_PATH = '/exports'

    global k8s
    k8s = KubeHelper()

    global EKS_CLUSTER_NAME
    global EKS_REGION
    global EKS_AUTO_SCALING_GROUP
    global EKS_SCALING_SIZE
    global EKS1_AUTO_SCALING_GROUP
    global EKS1_SCALING_SIZE
    EKS_CLUSTER_NAME = os.getenv('EKS_CLUSTER_NAME')
    EKS_REGION = os.getenv('EKS_REGION')
    EKS_AUTO_SCALING_GROUP = os.getenv('EKS_AUTO_SCALING_GROUP')
    EKS_SCALING_SIZE = os.getenv('EKS_SCALING_SIZE')
    EKS1_AUTO_SCALING_GROUP = os.getenv('EKS1_AUTO_SCALING_GROUP')
    EKS1_SCALING_SIZE = os.getenv('EKS1_SCALING_SIZE')

    global EFS_SERVER
    global EFS_PATH
    global EFS_ROOT
    global EFS_POD
    global EFS_EXTEND
    global EFS_SERVER_ID
    EFS_SERVER = os.getenv('EFS_SERVER')
    EFS_PATH = os.getenv('EFS_PATH')
    EFS_ROOT = os.getenv('EFS_ROOT')
    EFS_POD = os.getenv('EFS_POD')
    EFS_EXTEND = os.getenv('EFS_EXTEND')
    EFS_SERVER_ID = os.getenv('EFS_SERVER_ID')

    global RCA_NAME
    global RCA_DOMAIN
    RCA_NAME = os.getenv('RCA_NAME')
    RCA_DOMAIN = os.getenv('RCA_DOMAIN')

    global ORDERER_ORGS
    global ORDERER_DOMAINS
    global ORDERER_TYPE
    ORDERER_ORGS = os.getenv('ORDERER_ORGS')
    ORDERER_DOMAINS = os.getenv('ORDERER_DOMAINS')
    ORDERER_TYPE = os.getenv('ORDERER_TYPE')

    global KAFKA_NAMESPACE
    KAFKA_NAMESPACE = os.getenv('KAFKA_NAMESPACE')

    global BATCH_TIMEOUT
    global BATCH_SIZE_MAX_MESSAGE_COUNT
    BATCH_TIMEOUT = os.getenv('BATCH_TIMEOUT')
    BATCH_SIZE_MAX_MESSAGE_COUNT = os.getenv('BATCH_SIZE_MAX_MESSAGE_COUNT')

    global NUM_ORDERERS
    NUM_ORDERERS = os.getenv('NUM_ORDERERS')

    global PEER_ORGS
    global PEER_DOMAINS
    global PEER_PREFIX
    PEER_ORGS = os.getenv('PEER_ORGS')
    PEER_DOMAINS = os.getenv('PEER_DOMAINS')
    PEER_PREFIX = os.getenv('PEER_PREFIX')

    global NUM_PEERS
    NUM_PEERS = os.getenv('NUM_PEERS')

    global CHANNEL_NAME
    CHANNEL_NAME = os.getenv('CHANNEL_NAME')

    global PRIVATE_DOCKER_IMAGE
    global PRIVATE_DOCKER_SEVER
    global PRIVATE_DOCKER_USER
    global PRIVATE_DOCKER_PASSWORD
    global PRIVATE_DOCKER_EMAIL
    PRIVATE_DOCKER_IMAGE = os.getenv('PRIVATE_DOCKER_IMAGE')
    PRIVATE_DOCKER_SEVER = os.getenv('PRIVATE_DOCKER_SEVER')
    PRIVATE_DOCKER_USER = os.getenv('PRIVATE_DOCKER_USER')
    PRIVATE_DOCKER_PASSWORD = os.getenv('PRIVATE_DOCKER_PASSWORD')
    PRIVATE_DOCKER_EMAIL = os.getenv('PRIVATE_DOCKER_EMAIL')

    global FABRIC_TAG
    FABRIC_TAG = os.getenv('FABRIC_TAG')

    global EXTERNAL_ORDERER_ADDRESSES, EXTERNAL_ORG_PEER0_ADDRESSES
    global EXTERNAL_ORG_PEER1_ADDRESSES, EXTERNAL_RCA_ADDRESSES
    EXTERNAL_ORDERER_ADDRESSES = os.getenv('EXTERNAL_ORDERER_ADDRESSES')
    EXTERNAL_ORG_PEER0_ADDRESSES = os.getenv('EXTERNAL_ORG_PEER0_ADDRESSES')
    EXTERNAL_ORG_PEER1_ADDRESSES = os.getenv('EXTERNAL_ORG_PEER1_ADDRESSES')
    EXTERNAL_RCA_ADDRESSES = os.getenv('EXTERNAL_RCA_ADDRESSES')

    global NEW_ORG_NAME
    NEW_ORG_NAME = os.getenv('NEW_ORG_NAME')

    global ORGS, DOMAINS
    ORGS = (ORDERER_ORGS+' ' +PEER_ORGS).strip()
    DOMAINS = (ORDERER_DOMAINS+' '+PEER_DOMAINS).strip()



import click
import yaml
import re
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def enroll_orderer(orderer):
    # Get domain
    domain = util.get_domain(orderer)

    # Create temp folder & namespace
    settings.k8s.prereqs(domain)

    k8s_template_file = '%s/enroll-orderer/fabric-deployment-enroll-orderer.yaml' % util.get_k8s_template_path()
    dict_env = {
        'ORDERER': orderer,
        'ENROLL_DOMAIN': domain,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }

    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=k8s_template_file, dict_env=dict_env)

def del_enroll_orderer(orderer):

    # Get domain
    domain = util.get_domain(orderer)
    jobname = 'enroll-o-%s' % orderer

    # Delete job pod
    return settings.k8s.delete_job(name=jobname, namespace=domain)

def enroll_all_orderer():
    orderers = settings.ORDERER_ORGS.split(' ')
    # TODO: Multiprocess
    for orderer in orderers:
        enroll_orderer(orderer)

def del_all_enroll_orderer():
    orderers = settings.ORDERER_ORGS.split(' ')
    # TODO: Multiprocess
    for orderer in orderers:
        del_enroll_orderer(orderer)

@click.group()
def enroll_orderers():
    """Enroll orderers"""
    pass

@enroll_orderers.command('setup', short_help="Setup enroll orderers")
def setup():
    hiss.rattle('Enroll orderers')
    enroll_all_orderer()

@enroll_orderers.command('delete', short_help="Delete enroll orderers")
def delete():
    hiss.rattle('Delete enroll orderers job')
    del_all_enroll_orderer()



import click
import os
from settings import settings
from os import path
from os.path import expanduser
from shutil import copyfile
from utils import hiss


def copy_scripts():
    hiss.rattle('Copy scripts to EFS')

    # Find efs pod
    pods = settings.k8s.find_pod(namespace="default", keyword="test-efs")
    if not pods:
        return hiss.hiss('cannot find tiller pod')

    # Check empty folder
    exec_command = [
        '/bin/bash',
        '-c',
        'test -d %s && echo "1" || echo "0"'  % (settings.EFS_ROOT)]

    result_get_folder = settings.k8s.exec_pod(
        podName=pods[0], namespace="default", command=exec_command)
    if int(result_get_folder.data) < 1:
        hiss.sub_echo('Folder %s not found. Creating...' % settings.EFS_ROOT)
        exec_command = [
            '/bin/bash',
            '-c',
            'mkdir -p %s/admin; mkdir -p %s/akc-ca-data' % (settings.EFS_ROOT, settings.EFS_ROOT)]

        # Create folder in efs
        result_create_folder = settings.k8s.exec_pod(
            podName=pods[0], namespace="default", command=exec_command)
        if result_create_folder.success == False:
            return hiss.hiss('cannot create folders in %s: %s' % (pods[0], result_create_folder.msg))

    # Copy config to scripts/env
    hiss.sub_echo('Copy config to scripts/env')
    config_file = expanduser('~/.akachain/akc-mamba/mamba/config/.env')
    env_script_File = expanduser('~/.akachain/akc-mamba/mamba/blockchain/scripts/env-scripts.sh')
    copyfile(config_file, env_script_File)

    # Remove old script folder in efs
    hiss.sub_echo('Remove old script folder in efs')
    exec_command = [
        '/bin/bash',
        '-c',
        'rm -rf %s/akc-ca-scripts/*' % (settings.EFS_ROOT)]

    result_create_folder = settings.k8s.exec_pod(
        podName=pods[0], namespace="default", command=exec_command)
    if result_create_folder.success == False:
        return hiss.hiss('cannot remove folders in %s' % pods[0])

    # Copy scripts folder to efs
    hiss.sub_echo('Copy scripts folder to efs')
    script_path = expanduser('~/.akachain/akc-mamba/mamba/blockchain/scripts')
    if not settings.k8s.cp_to_pod(podName=pods[0], namespace='default', source=script_path, target='%s/akc-ca-scripts' % settings.EFS_ROOT):
        return hiss.hiss('connot copy scripts folder to pod %s' % pods[0])

    exec_command = [
        '/bin/bash',
        '-c',
        ('test -d '+settings.EFS_ROOT+'/akc-ca-scripts/akc-ca-scripts'
        ' && mv '+settings.EFS_ROOT+'/akc-ca-scripts/akc-ca-scripts/* '+settings.EFS_ROOT+'/akc-ca-scripts || echo "ok"')]

    result_create_folder = settings.k8s.exec_pod(
        podName=pods[0], namespace="default", command=exec_command)
    if result_create_folder.success == False:
        return hiss.hiss('cannot remove folders in %s' % pods[0])

    # Copy test chaincode to efs
    hiss.sub_echo('Copy test chaincode to efs')
    artifacts_path = expanduser('~/.akachain/akc-mamba/mamba/blockchain/artifacts')
    if not settings.k8s.cp_to_pod(podName=pods[0], namespace='default', source=artifacts_path, target='%s/admin/artifacts' % settings.EFS_ROOT):
        return hiss.hiss('connot copy test chaincode to pod %s' % pods[0])

    return True


@click.command('copyscripts', short_help="Copy \'scripts\' & \'chaincode test' folder to EFS")
def copyscripts():
    copy_scripts()



# A snake must know how hiss ... or sometimes rattle
# Normally we can just use echo to print out message during execution
# However:
    # It is mandatory to *hiss* when there is error
    # also, *rattle* is needed when a snake meet something ... at the beginning
    # or at the end of an execution.
import click

def echo(message):
    click.echo(message)

def sub_echo(message):
    click.echo('  ' + message)

def hiss(message):
    click.secho('Error: ' + message, fg='red')
    return False

def rattle(message):
    click.secho(message, blink=True, bold=True, fg='white', bg='blue')


import click

@click.group()
def vpn():
    pass

@vpn.command('init', short_help="Initialize vpn service")
def init():
    click.echo('Initialize vpn service')

@vpn.command('drop', short_help="Remove vpn service")
def drop():
    click.echo('Remove vpn service')


import click
import yaml
import re
import json
import os
import time
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def get_namespace():
    # Get domain
    domains = settings.ORDERER_DOMAINS.split(' ')
    if len(domains) == 0:
        domains = settings.PEER_DOMAINS.split(' ')
    explorer_namespace = domains[0]
    
    # Create temp folder & namespace
    settings.k8s.prereqs(explorer_namespace)
    return explorer_namespace

def setup_grafana():

    hiss.echo('Deploy grafana')
    namespace = get_namespace()

    # Create temp folder & namespace
    settings.k8s.prereqs(namespace)

    dict_env = {
        'DOMAIN': namespace
    }

    # Deploy grafana sts
    grafana_template = '%s/grafana/grafana-stateful.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=namespace, k8s_template_file=grafana_template, dict_env=dict_env)
    # Deploy grafana svc
    grafana_sv_template = '%s/grafana/grafana-service-stateful.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=namespace, k8s_template_file=grafana_sv_template, dict_env=dict_env)

def del_grafana():
    # Delete
    return settings.k8s.delete_stateful(name='grafana', namespace=get_namespace())

@click.group()
def grafana():
    """Grafana"""
    pass

@grafana.command('setup', short_help="Setup grafana")
def setup():

    hiss.rattle('Setup grafana')
    setup_grafana()

@grafana.command('delete', short_help="Delete grafana")
def delete():

    hiss.rattle('Delete grafana')
    del_grafana()



import click
import os
import time
from settings import settings
from os import path

from shutil import copyfile
from utils import hiss

from blockchain.copyscripts.commands import copy_scripts
from blockchain.rca.commands import setup_rca
from blockchain.ica.commands import setup_all_ica
from blockchain.reg_orgs.commands import reg_all_org
from blockchain.reg_orderers.commands import reg_all_orderer
from blockchain.reg_peers.commands import reg_all_peer
from blockchain.enroll_orderers.commands import enroll_all_orderer
from blockchain.enroll_peers.commands import enroll_all_peer
from blockchain.update_folder.commands import update_folder
from blockchain.zookeeper.commands import setup_zookeeper
from blockchain.kafka.commands import setup_kafka
from blockchain.channel_artifact.commands import gen_channel_artifact
from blockchain.orderer.commands import setup_all_orderer
from blockchain.peer.commands import setup_all_peer
from blockchain.gen_artifact.commands import generate_artifact
from k8s.secret.commands import create_all_docker_secret
from blockchain.admin.commands import setup_admin
from blockchain.bootstrap_network.commands import bootstrap_network


def start_network():
    hiss.rattle('Setup Network')

    # Copy scripts to EFS
    copy_scripts()

    # Create a new Root Certificate Authority service
    setup_rca()

    # Create new Intermediate Certificate Authority services
    setup_all_ica()

    # Run jobs to register organizations
    reg_all_org()

    # Run jobs to register orderers
    reg_all_orderer()

    # Run jobs to register peers
    reg_all_peer()

    # Run jobs to enroll orderers
    enroll_all_orderer()

    # Run jobs to enroll peers
    enroll_all_peer()

    time.sleep(5)

    # Create crypto-config folder to contains artifacts
    update_folder()

    if settings.ORDERER_TYPE == 'kafka':
        # Create new Zookeeper services
        setup_zookeeper()
        # Create new Kafka services
        setup_kafka()
    
    # Run job to generate channel.tx, genesis.block
    gen_channel_artifact()

    # Create new StatefullSet orderers
    setup_all_orderer()

    # Create new StatefullSet peers
    setup_all_peer()

    # Run jobs to generate application artifacts
    generate_artifact()

    # Create secret if use private docker hub
    if settings.PRIVATE_DOCKER_IMAGE == 'true':
        create_all_docker_secret('mamba')

    # Create new a new Admin service
    time.sleep(1)
    setup_admin()

    # Bootrap network
    time.sleep(1)
    bootstrap_network()

    # cat log
    domains = settings.ORDERER_DOMAINS.split(' ')
    settings.k8s.read_pod_log('bootstrap-network', domains[0])

    return True


@click.command('start', short_help="Start Blockchain Network")
def start():
    start_network()


import click
import os
from settings import settings
from os import path
from shutil import copyfile
from utils import hiss, util


def update_folder():
    hiss.rattle('Update folder crt in EFS')

    # Find efs pod
    pods = settings.k8s.find_pod(namespace="default", keyword="test-efs")
    if not pods:
        return hiss.hiss('cannot find tiller pod')

    all_command = ''

    prepare_cmd = 'rm -rf %s/akc-ca-data/crypto-config;' % settings.EFS_ROOT
    prepare_cmd += 'cd %s/akc-ca-data/;'% settings.EFS_ROOT
    all_command += prepare_cmd

    if settings.ORDERER_ORGS != '':
        # Build orderer command
        orderers = settings.ORDERER_ORGS.split(' ')
        orderer_cmd = ''

        for orderer in orderers:
            # Get domain
            domain = util.get_domain(orderer) 
            orderer_cmd += (''
                'mkdir -p crypto-config/ordererOrganizations/'+domain+'/ca;'
                'mkdir -p crypto-config/ordererOrganizations/'+domain+'/msp/admincerts;'
                'mkdir -p crypto-config/ordererOrganizations/'+domain+'/msp/cacerts;'
                'mkdir -p crypto-config/ordererOrganizations/'+domain+'/msp/tlscacerts;'
                'mkdir -p crypto-config/ordererOrganizations/'+domain+'/tlsca;'
                'mkdir -p crypto-config/ordererOrganizations/'+domain+'/users/admin/msp/admincerts;'
                'mkdir -p crypto-config/ordererOrganizations/'+domain+'/users/admin/msp/cacerts;'
                'mkdir -p crypto-config/ordererOrganizations/'+domain+'/users/admin/msp/keystore;'
                'mkdir -p crypto-config/ordererOrganizations/'+domain+'/users/admin/msp/signcerts;'
                'mkdir -p crypto-config/ordererOrganizations/'+domain+'/users/admin/msp/tlscacerts;'
                'mkdir -p crypto-config/ordererOrganizations/'+domain+'/users/admin/tls;'
            '')
            for index in range(int(settings.NUM_ORDERERS)):
                orderer_cmd += (''
                    'mkdir -p crypto-config/ordererOrganizations/'+domain+'/orderers/orderer'+str(index)+'-'+orderer+'.'+domain+'/msp/admincerts;'
                    'mkdir -p crypto-config/ordererOrganizations/'+domain+'/orderers/orderer'+str(index)+'-'+orderer+'.'+domain+'/msp/cacerts;'
                    'mkdir -p crypto-config/ordererOrganizations/'+domain+'/orderers/orderer'+str(index)+'-'+orderer+'.'+domain+'/msp/keystore;'
                    'mkdir -p crypto-config/ordererOrganizations/'+domain+'/orderers/orderer'+str(index)+'-'+orderer+'.'+domain+'/msp/signcerts;'
                    'mkdir -p crypto-config/ordererOrganizations/'+domain+'/orderers/orderer'+str(index)+'-'+orderer+'.'+domain+'/msp/tlscacerts;'
                    'mkdir -p crypto-config/ordererOrganizations/'+domain+'/orderers/orderer'+str(index)+'-'+orderer+'.'+domain+'/tls;'

                    'cp ica-'+orderer+'-ca-chain.pem crypto-config/ordererOrganizations/'+domain+'/orderers/orderer'+str(index)+'-'+orderer+'.'+domain+'/msp/cacerts/ca.'+domain+'-cert.pem;'
                    'cp ica-'+orderer+'-ca-chain.pem crypto-config/ordererOrganizations/'+domain+'/orderers/orderer'+str(index)+'-'+orderer+'.'+domain+'/msp/tlscacerts/tlsca.'+domain+'-cert.pem;'
                    'cp ica-'+orderer+'-ca-chain.pem crypto-config/ordererOrganizations/'+domain+'/orderers/orderer'+str(index)+'-'+orderer+'.'+domain+'/tls/tlsca.'+domain+'-cert.pem;'
                    'cp orgs/'+orderer+'/admin/msp/admincerts/cert.pem crypto-config/ordererOrganizations/'+domain+'/orderers/orderer'+str(index)+'-'+orderer+'.'+domain+'/msp/admincerts/cert.pem;'
                    'cp orgs/orderer/msp/signcerts/cert.pem crypto-config/ordererOrganizations/'+domain+'/orderers/orderer'+str(index)+'-'+orderer+'.'+domain+'/msp/signcerts/;'
                    'cp orgs/orderer/msp/keystore/key.pem crypto-config/ordererOrganizations/'+domain+'/orderers/orderer'+str(index)+'-'+orderer+'.'+domain+'/msp/keystore/key.pem;'
                    'cp orgs/orderer/tls/server.crt crypto-config/ordererOrganizations/'+domain+'/orderers/orderer'+str(index)+'-'+orderer+'.'+domain+'/tls/;'
                    'cp orgs/orderer/tls/server.key crypto-config/ordererOrganizations/'+domain+'/orderers/orderer'+str(index)+'-'+orderer+'.'+domain+'/tls/;'
                '')
            orderer_cmd += (''
                'cp ica-'+orderer+'-ca-chain.pem crypto-config/ordererOrganizations/'+domain+'/ca/ca.'+domain+'-cert.pem;'
                'cp ica-'+orderer+'-ca-chain.pem crypto-config/ordererOrganizations/'+domain+'/msp/cacerts/ca.'+domain+'-cert.pem;'
                'cp ica-'+orderer+'-ca-chain.pem crypto-config/ordererOrganizations/'+domain+'/users/admin/msp/cacerts/ca.'+domain+'-cert.pem;'
                'cp ica-'+orderer+'-ca-chain.pem crypto-config/ordererOrganizations/'+domain+'/tlsca/tlsca.'+domain+'-cert.pem;'
                'cp ica-'+orderer+'-ca-chain.pem crypto-config/ordererOrganizations/'+domain+'/msp/tlscacerts/tlsca.'+domain+'-cert.pem;'
                'cp ica-'+orderer+'-ca-chain.pem crypto-config/ordererOrganizations/'+domain+'/users/admin/msp/tlscacerts/tlsca.'+domain+'-cert.pem;'
                'cp ica-'+orderer+'-ca-chain.pem crypto-config/ordererOrganizations/'+domain+'/users/admin/tls/tlsca.'+domain+'-cert.pem;'
                'cp orgs/'+orderer+'/admin/msp/admincerts/cert.pem crypto-config/ordererOrganizations/'+domain+'/msp/admincerts/cert.pem;'
                'cp orgs/'+orderer+'/admin/msp/admincerts/cert.pem crypto-config/ordererOrganizations/'+domain+'/users/admin/msp/admincerts/cert.pem;'
                'cp orgs/'+orderer+'/admin/msp/keystore/* crypto-config/ordererOrganizations/'+domain+'/users/admin/msp/keystore/key.pem;'
                'cp orgs/'+orderer+'/admin/msp/signcerts/cert.pem crypto-config/ordererOrganizations/'+domain+'/users/admin/msp/signcerts/cert.pem;'
                'echo "succeed";'
            '')
        all_command += orderer_cmd

    # Build peer command
    peers = settings.PEER_ORGS.split(' ')
    peer_cmd = ''
    for peer in peers:
        # Get domain
        domain = util.get_domain(peer)
        peer_cmd += (''
            'mkdir -p crypto-config/peerOrganizations/'+domain+'/ca;'
            'mkdir -p crypto-config/peerOrganizations/'+domain+'/msp/admincerts;'
            'mkdir -p crypto-config/peerOrganizations/'+domain+'/msp/cacerts;'
            'mkdir -p crypto-config/peerOrganizations/'+domain+'/msp/tlscacerts;'
            'mkdir -p crypto-config/peerOrganizations/'+domain+'/tlsca;'
            'mkdir -p crypto-config/peerOrganizations/'+domain+'/users/admin/msp/admincerts;'
            'mkdir -p crypto-config/peerOrganizations/'+domain+'/users/admin/msp/cacerts;'
            'mkdir -p crypto-config/peerOrganizations/'+domain+'/users/admin/msp/keystore;'
            'mkdir -p crypto-config/peerOrganizations/'+domain+'/users/admin/msp/signcerts;'
            'mkdir -p crypto-config/peerOrganizations/'+domain+'/users/admin/msp/tlscacerts;'
            'mkdir -p crypto-config/peerOrganizations/'+domain+'/users/admin/tls;'
        '')
        for index in range(int(settings.NUM_PEERS)):
            peer_cmd += (''
                'mkdir -p crypto-config/peerOrganizations/'+domain+'/peers/peer'+str(index)+'.'+domain+'/msp/admincerts;'
                'mkdir -p crypto-config/peerOrganizations/'+domain+'/peers/peer'+str(index)+'.'+domain+'/msp/cacerts;'
                'mkdir -p crypto-config/peerOrganizations/'+domain+'/peers/peer'+str(index)+'.'+domain+'/msp/keystore;'
                'mkdir -p crypto-config/peerOrganizations/'+domain+'/peers/peer'+str(index)+'.'+domain+'/msp/signcerts;'
                'mkdir -p crypto-config/peerOrganizations/'+domain+'/peers/peer'+str(index)+'.'+domain+'/msp/tlscacerts;'
                'mkdir -p crypto-config/peerOrganizations/'+domain+'/peers/peer'+str(index)+'.'+domain+'/tls;'

                'cp ica-'+peer+'-ca-chain.pem crypto-config/peerOrganizations/'+domain+'/peers/peer'+str(index)+'.'+domain+'/msp/cacerts/ca.'+domain+'-cert.pem;'
                'cp ica-'+peer+'-ca-chain.pem crypto-config/peerOrganizations/'+domain+'/peers/peer'+str(index)+'.'+domain+'/msp/tlscacerts/tlsca.'+domain+'-cert.pem;'
                'cp ica-'+peer+'-ca-chain.pem crypto-config/peerOrganizations/'+domain+'/peers/peer'+str(index)+'.'+domain+'/tls/tlsca.'+domain+'-cert.pem;'
                'cp orgs/'+peer+'/admin/msp/admincerts/cert.pem crypto-config/peerOrganizations/'+domain+'/peers/peer'+str(index)+'.'+domain+'/msp/admincerts/cert.pem;'
                'cp orgs/peer'+str(index)+'-'+peer+'/msp/keystore/cert.pem crypto-config/peerOrganizations/'+domain+'/peers/peer'+str(index)+'.'+domain+'/msp/signcerts/;'
                'cp orgs/peer'+str(index)+'-'+peer+'/msp/keystore/key.pem crypto-config/peerOrganizations/'+domain+'/peers/peer'+str(index)+'.'+domain+'/msp/keystore/key.pem;'
                'cp orgs/peer'+str(index)+'-'+peer+'/tls/server.crt crypto-config/peerOrganizations/'+domain+'/peers/peer'+str(index)+'.'+domain+'/tls/;'
                'cp orgs/peer'+str(index)+'-'+peer+'/tls/server.key crypto-config/peerOrganizations/'+domain+'/peers/peer'+str(index)+'.'+domain+'/tls/;'
            '')
        peer_cmd += (''
            'cp ica-'+peer+'-ca-chain.pem crypto-config/peerOrganizations/'+domain+'/ca/ca.'+domain+'-cert.pem;'
            'cp ica-'+peer+'-ca-chain.pem crypto-config/peerOrganizations/'+domain+'/msp/cacerts/ca.'+domain+'-cert.pem;'
            'cp ica-'+peer+'-ca-chain.pem crypto-config/peerOrganizations/'+domain+'/users/admin/msp/cacerts/ca.'+domain+'-cert.pem;'
            'cp ica-'+peer+'-ca-chain.pem crypto-config/peerOrganizations/'+domain+'/tlsca/tlsca.'+domain+'-cert.pem;'
            'cp ica-'+peer+'-ca-chain.pem crypto-config/peerOrganizations/'+domain+'/msp/tlscacerts/tlsca.'+domain+'-cert.pem;'
            'cp ica-'+peer+'-ca-chain.pem crypto-config/peerOrganizations/'+domain+'/users/admin/msp/tlscacerts/tlsca.'+domain+'-cert.pem;'
            'cp ica-'+peer+'-ca-chain.pem crypto-config/peerOrganizations/'+domain+'/users/admin/tls/tlsca.'+domain+'-cert.pem;'
            'cp orgs/'+peer+'/admin/msp/admincerts/cert.pem crypto-config/peerOrganizations/'+domain+'/msp/admincerts/cert.pem;'
            'cp orgs/'+peer+'/admin/msp/admincerts/cert.pem crypto-config/peerOrganizations/'+domain+'/users/admin/msp/admincerts/cert.pem;'
            'cp orgs/'+peer+'/admin/msp/keystore/* crypto-config/peerOrganizations/'+domain+'/users/admin/msp/keystore/key.pem;'
            'cp orgs/'+peer+'/admin/msp/signcerts/cert.pem crypto-config/peerOrganizations/'+domain+'/users/admin/msp/signcerts/cert.pem;'
        '')
    all_command += peer_cmd

    # Exec command
    exec_command = [
        '/bin/bash',
        '-c',
        '%s'  % (all_command)]

    result_get_folder = settings.k8s.exec_pod(
        podName=pods[0], namespace="default", command=exec_command)
    hiss.sub_echo(result_get_folder.data)
    return True


@click.command('updatefolder', short_help="Update folder crypto-config in EFS")
def updatefolder():
    update_folder()



import click
import os
import time
from settings import settings
from os import path
from shutil import copyfile
from utils import hiss

from blockchain.rca.commands import delete_rca
from blockchain.ica.commands import delete_all_ica
from blockchain.reg_orgs.commands import del_all_reg_org
from blockchain.reg_orderers.commands import del_all_reg_orderer
from blockchain.reg_peers.commands import del_all_reg_peer
from blockchain.enroll_orderers.commands import del_all_enroll_orderer
from blockchain.enroll_peers.commands import del_all_enroll_peer
from blockchain.zookeeper.commands import delete_zookeeper
from blockchain.kafka.commands import delete_kafka
from blockchain.channel_artifact.commands import del_gen_channel_artifact
from blockchain.orderer.commands import delete_all_orderer
from blockchain.peer.commands import del_all_peer
from blockchain.gen_artifact.commands import del_generate_artifact
from blockchain.admin.commands import del_admin
from blockchain.bootstrap_network.commands import del_bootstrap_network


def delete_network():
    hiss.rattle('Delete Network')

    # Delete Root Certificate Authority service
    delete_rca()

    # Delete Intermediate Certificate Authority services
    delete_all_ica()

    # Delete jobs to register organizations
    del_all_reg_org()

    # Delete jobs to register orderers
    del_all_reg_orderer()

    # Delete jobs to register peers
    del_all_reg_peer()

    # Delete jobs to enroll orderers
    del_all_enroll_orderer()

    # Delete jobs to enroll peers
    del_all_enroll_peer()


    if settings.ORDERER_TYPE == 'kafka':
        # Delete Zookeeper services
        delete_zookeeper()
        # Delete Kafka services
        delete_kafka()
    
    # Delete job to generate channel.tx, genesis.block
    del_gen_channel_artifact()

    # Delete StatefullSet orderers
    delete_all_orderer()

    # Delete StatefullSet peers
    del_all_peer()

    # Delete jobs to generate application artifacts
    del_generate_artifact()

    # Delete Admin service
    del_admin()

    del_bootstrap_network()

    return True


@click.command('delete', short_help="Delete Blockchain Network")
def delete():
    delete_network()



import click
import os
import time
from settings import settings
from os import path

from shutil import copyfile
from utils import hiss, util

from blockchain.copyscripts.commands import copy_scripts
from blockchain.rca.commands import setup_rca
from blockchain.ica.commands import setup_all_ica
from blockchain.reg_orgs.commands import reg_all_org
from blockchain.reg_orderers.commands import reg_all_orderer
from blockchain.reg_peers.commands import reg_all_peer
from blockchain.enroll_orderers.commands import enroll_all_orderer
from blockchain.enroll_peers.commands import enroll_all_peer
from blockchain.update_folder.commands import update_folder
from blockchain.zookeeper.commands import setup_zookeeper
from blockchain.kafka.commands import setup_kafka
from blockchain.channel_artifact.commands import gen_channel_artifact
from blockchain.orderer.commands import setup_all_orderer
from blockchain.peer.commands import setup_all_peer
from blockchain.gen_artifact.commands import generate_artifact
from k8s.secret.commands import create_docker_secret
from blockchain.admin.commands import setup_admin
from blockchain.bootstrap_network.commands import bootstrap_network


def create_new_org():
    hiss.rattle('Create New Org')

    # Copy scripts to EFS
    copy_scripts()

    # Phai co rca external address
    # Create new Intermediate Certificate Authority services
    setup_all_ica()

    # Run jobs to register organizations
    reg_all_org()

    # Run jobs to register peers
    reg_all_peer()

    # Run jobs to enroll peers
    enroll_all_peer()

    time.sleep(5)

    # Create crypto-config folder to contains artifacts
    update_folder()

    # Default value of ORDERER_DOMAINS = default, it needed to run processes below
    if (settings.ORDERER_DOMAINS == ''):
        settings.ORDERER_DOMAINS='default'

    # Create new StatefullSet peers
    setup_all_peer()

    # Run jobs to generate application artifacts
    generate_artifact()

    # Create secret if use private docker hub
    if settings.PRIVATE_DOCKER_IMAGE == 'true':
        create_docker_secret('default','mamba')

    # Create new a new Admin service
    time.sleep(1)
    setup_admin()

    # # Return value ORDERER_DOMAINS
    if (settings.ORDERER_DOMAINS == 'default'):
        settings.ORDERER_DOMAINS=''

    dict_env = {
        'ORG_NAME': settings.PEER_ORGS,
        'ORG_DOMAIN': settings.PEER_DOMAINS,
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }

    # Create configtx
    create_config_template = '%s/add-org/0create-configtx.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=settings.PEER_DOMAINS, k8s_template_file=create_config_template, dict_env=dict_env)

    # Gen org.json
    gen_artifact_template = '%s/add-org/1gen-artifacts.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=settings.PEER_DOMAINS, k8s_template_file=gen_artifact_template, dict_env=dict_env)

    return True


@click.command('create-org', short_help="Start Blockchain Network")
def create_org():
    create_new_org()


import os
from os.path import expanduser
import datetime
import json
import re
import yaml
from utils import hiss
from settings import settings

class Result(object):
    def __init__(self, success, msg=None, data=None):
        self.success = success
        self.msg = msg
        self.data = data
    def __str__(self):
        return '{\"success\": \"%s\", \"msg\": \"%s\", \"data\":\"%s\\"}' % (self.success, self.msg, self.data)

def load_yaml_config_template(yaml_template_path, dict_env):
    new_yaml_file_name = yaml_template_path.split('/').pop()
    # Load template file
    with open(yaml_template_path, 'r') as sources:
        lines = sources.readlines()
        out_data = []
    # Replace variable
    for line in lines:
        out_line = line
        for key, value in dict_env.items():
            out_line = re.sub(r'{{%s}}' % key, value, out_line)
        out_data.append(out_line)
    # Get time now
    current_time = split_timenow_utc()

     # Create yaml_path
    hiss.echo('Create yaml file')
    yaml_path = '%s/%s/%s_%s' % (get_temp_path(),
                                     current_time[0], current_time[1], new_yaml_file_name)
    # Write yaml -> yaml_path
    with open(yaml_path, "w") as sources:
        for line in out_data:
            sources.write(line)

    # Read new yaml
    with open(yaml_path, 'r') as ymlfile:
        config = yaml.load_all(ymlfile)
    return yaml_path, config

def smart_append(list_item, items):
    if isinstance(items, list):
        list_item += items
    else:
        list_item.append(items)
    return list_item


def get_temp_path():
    return expanduser('~/.akachain/akc-mamba/mamba/_temp')


def get_k8s_template_path():
    return expanduser('~/.akachain/akc-mamba/mamba/blockchain/template')


def split_timenow_utc():
    # Get current datetime (UTC)
    current_time = datetime.datetime.utcnow().replace(
        microsecond=0).isoformat().split('T')

    current_time[1] = current_time[1].replace(':','')
    return current_time


def make_folder(path_folder):
    if not os.path.exists(path_folder):
        hiss.sub_echo('Folder %s does not exists. \n\tCreating...' %
                      path_folder)
        os.mkdir(path_folder)
    else:
        hiss.sub_echo('Folder temp %s exists.' % path_folder)


def make_temp_folder():
    hiss.echo('Create Folder temp')
    temp_path = get_temp_path()
    make_folder(temp_path)

    # Get current time
    current_time = split_timenow_utc()

    # Make folder temp if it not exists
    yaml_path = '%s/%s' % (temp_path, current_time[0])
    make_folder(yaml_path)


def get_domain(org_name):
    orgs = settings.ORGS.split(' ')
    domains = settings.DOMAINS.split(' ')

    if org_name in orgs:
        return domains[orgs.index(org_name)]
    else:
        return hiss.hiss('org_name: %s does not exists in env file' % org_name)

def get_peer_external_domain(peer, index_peer):
    peers = settings.PEER_ORGS.split(' ')

    if index_peer == 0:
        ex_domains = settings.EXTERNAL_ORG_PEER0_ADDRESSES.split(' ')
    else:
        ex_domains = settings.EXTERNAL_ORG_PEER1_ADDRESSES.split(' ')

    if peer in peers:
        if len(ex_domains) > peers.index(peer):
            return ex_domains[peers.index(peer)]
        else:
            return ''
    else:
        return hiss.hiss('peer: %s does not exists in env file' % peer)



import setuptools
import sys

with open("README.md", "r") as fh:
  long_description = fh.read()

setuptools.setup(
  name='akc-mamba',
  version='0.0.5',
  entry_points={'console_scripts': ['mamba = mamba.mamba:main'] },
  author="akaChain",
  author_email="admin@akachain.io",
  description="A production ready, complete experience in deploying a Hyperledger Fabric",
  long_description=long_description,
  long_description_content_type="text/markdown",
  url="https://github.com/Akachain/akc-mamba",
  include_package_data=True,
  package_dir={'': 'mamba'},
  packages=setuptools.find_packages('mamba'),
  classifiers=[
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
  ],
)



import click
import yaml
import re
import datetime
from kubernetes import client
from os import path
from kubernetes.client.rest import ApiException
from utils import hiss, util
from settings import settings


def terminate_zookeeper():
    domain = settings.KAFKA_NAMESPACE
    name = 'zoo'

    # Terminate stateful set
    return settings.k8s.delete_stateful(name=name, namespace=domain, delete_pvc=True)

def delete_zookeeper():
    domain = settings.KAFKA_NAMESPACE
    name = 'zoo'

    # Delete stateful set
    return settings.k8s.delete_stateful(name=name, namespace=domain)

def setup_zookeeper():

    domain = settings.KAFKA_NAMESPACE

    # Create temp folder & namespace
    settings.k8s.prereqs(domain)

    dict_env = {
        'KAFKA_NAMESPACE': domain
    }

    zk_cs_template_file = '%s/zookeeper/0zk-cs.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=zk_cs_template_file, dict_env=dict_env)

    zk_hs_template_file = '%s/zookeeper/1zk-hs.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=zk_hs_template_file, dict_env=dict_env)

    zk_set_template_file = '%s/zookeeper/2zk-set.yaml' % util.get_k8s_template_path()
    settings.k8s.apply_yaml_from_template(
        namespace=domain, k8s_template_file=zk_set_template_file, dict_env=dict_env)


@click.group()
def zookeeper():
    """Zookeeper"""
    pass


@zookeeper.command('setup', short_help="Setup Zookeeper")
def setup():
    hiss.rattle('Setup Zookeeper')
    setup_zookeeper()

@zookeeper.command('delete', short_help="Delete Zookeeper")
def delete():
    hiss.rattle('Delete Zookeeper')
    delete_zookeeper()

@zookeeper.command('terminate', short_help="Terminate Zookeeper")
def terminate():
    hiss.rattle('Terminate Zookeeper')
    terminate_zookeeper()



import click
import yaml
import re
from kubernetes import client
from os import path
from utils import hiss, util
from settings import settings

def generate_artifact():

    domains = settings.ORDERER_DOMAINS.split(' ')

    # Create temp folder & namespace
    settings.k8s.prereqs(domains[0])

    k8s_template_file = '%s/gen-artifacts/fabric-deployment-gen-artifacts.yaml' % util.get_k8s_template_path()
    dict_env = {
        'ORDERER_DOMAIN': domains[0],
        'EFS_SERVER': settings.EFS_SERVER,
        'EFS_PATH': settings.EFS_PATH,
        'EFS_EXTEND': settings.EFS_EXTEND,
        'PVS_PATH': settings.PVS_PATH
    }

    settings.k8s.apply_yaml_from_template(
        namespace=domains[0], k8s_template_file=k8s_template_file, dict_env=dict_env)

def del_generate_artifact():

    domains = settings.ORDERER_DOMAINS.split(' ')
    jobname = 'generate-artifacts'

    # Delete job pod
    return settings.k8s.delete_job(name=jobname, namespace=domains[0])
    

@click.group()
def gen_artifact():
    """Generate application artifacts"""
    pass

@gen_artifact.command('setup', short_help="Run job to generate application artifacts")
def setup():
    hiss.rattle('Generate application artifacts')

    generate_artifact()

@gen_artifact.command('delete', short_help="Delete job gen artifact")
def delete():
    hiss.rattle('Delete job gen artifact')

    del_generate_artifact()
